@@ -3,7 +3,7 @@ use std::{collections::BTreeMap, iter::repeat};
 use criterion::{black_box, criterion_group, criterion_main, Criterion};
 use evalexpr::{build_operator_tree, ContextWithMutableVariables, HashMapContext, Node, Value};
 use exmex::OwnedFlatEx;
-use exmex::{ops_factory, prelude::*, BinOp, Operator};
+use exmex::{ops_factory, prelude::*, BinOp, MakeOperators, Operator};
 use fasteval::{Compiler, Evaler, Instruction, Slab};
 use itertools::{izip, Itertools};
 
@@ -127,51 +127,49 @@ fn exmex_bench_parse(c: &mut Criterion) {
 ops_factory!(
     OnlyNeededOperators,
     f64,
-    Operator {
-        repr: "^",
-        bin_op: Some(BinOp {
+    Operator::make_bin(
+        "^",
+        BinOp {
             apply: |a: f64, b| a.powf(b),
-            prio: 2,
-        }),
-        unary_op: None,
-    },
-    Operator {
-        repr: "*",
-        bin_op: Some(BinOp {
+            prio: 4,
+            is_commutative: false
+        }
+    ),
+    Operator::make_bin(
+        "*",
+        BinOp {
             apply: |a, b| a * b,
-            prio: 1,
-        }),
-        unary_op: None,
-    },
-    Operator {
-        repr: "/",
-        bin_op: Some(BinOp {
+            prio: 2,
+            is_commutative: true
+        }
+    ),
+    Operator::make_bin(
+        "/",
+        BinOp {
             apply: |a, b| a / b,
-            prio: 1,
-        }),
-        unary_op: None,
-    },
-    Operator {
-        repr: "+",
-        bin_op: Some(BinOp {
+            prio: 3,
+            is_commutative: false
+        }
+    ),
+    Operator::make_bin_unary(
+        "+",
+        BinOp {
             apply: |a, b| a + b,
             prio: 0,
-        }),
-        unary_op: Some(|a| a),
-    },
-    Operator {
-        repr: "-",
-        bin_op: Some(BinOp {
+            is_commutative: true
+        },
+        |a| a
+    ),
+    Operator::make_bin_unary(
+        "-",
+        BinOp {
             apply: |a, b| a - b,
-            prio: 0,
-        }),
-        unary_op: Some(|a| (-a)),
-    },
-    Operator {
-        repr: "sin",
-        bin_op: None,
-        unary_op: Some(|a| a.sin()),
-    }
+            prio: 1,
+            is_commutative: false
+        },
+        |a| -a
+    ),
+    Operator::make_unary("sin", |a| a.sin())
 );
 
 fn exmex_parse_optimized<'a>(strings: &'a [&str]) -> Vec<FlatEx<'a, f64, OnlyNeededOperators>> {
