@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.10.0")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.11.0")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //!
@@ -8,17 +8,19 @@
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex;
-//! assert!((exmex::eval_str::<f64>("1.5 * ((cos(0) + 23.0) / 2.0)")? - 18.0).abs() < 1e-12);
+//! assert!((exmex::eval_str::<f64>("1.5 * ((cos(2*PI) + 23.0) / 2.0)")? - 18.0).abs() < 1e-12);
 //! #
 //! #     Ok(())
 //! # }
 //! ```
 //! For floats, we have a list of predifined operators containing
 //! `^`, `*`, `/`, `+`, `-`, `sin`, `cos`, `tan`, `exp`, `log`, and `log2`. The full list is
-//! defined in [`DefaultOpsFactory`](DefaultOpsFactory). Library users can also create their
-//! own operators as shown below in the section about extendability.
+//! defined in [`DefaultOpsFactory`](DefaultOpsFactory). Further, the constants π and Euler's number can be
+//! used via `PI` and `E`, respectively. Library users can also create their
+//! own operators and constants as shown below in the section about extendability.
 //!
 //! ## Variables
+//!
 //! To define variables we can use strings that are not in the list of operators as shown in the following expression.
 //! Additionally, variables should consist only of letters, numbers, and underscores. More precisely, they need to fit the
 //! regular expression
@@ -59,18 +61,32 @@
 //!
 //! ## Extendability
 //!
-//! How to use custom operators as well as custom data types of the operands even with 
+//! How to use custom operators as well as custom data types of the operands even with
 //! non-numeric literals is described in the following sub-sections.
 //!
-//! ### Custom Operators
+//! ### Custom Operators and Constants
 //!
 //! Operators are instances of the struct
-//! [`Operator`](Operator). An operator's representation in the string-to-be-parsed is defined in the field
-//! [`repr`](Operator::repr). Further, operator instances define a binary and a unary operator, since an operator 
-//! representation can correspond to both such as `-` or `+`. Note that we expect a unary operator to be always 
-//! on the left of a number.
+//! [`Operator`](Operator). Constants are also defined in terms of constant operators. More precisely, 
+//! operators can be
+//! * binary such as `*`,
+//! * unary such as `sin`,
+//! * binary as well as unary such as `-`, or
+//! * constant such as `PI`. 
 //!
-//! To make serialization via [`serde`](https://serde.rs/) possible, operators need to be created by factories as 
+//! An operator's representation is defined in the field
+//! [`repr`](Operator::repr). A token of the string-to-be-parsed is identified as operator if it matches the operator's 
+//! representation exactly. For instance, `PI` will be parsed as the constant π while `PI5` will be parsed as a variable with name `PI5`.
+//! When an operator's representation is used in a string-to-be-parsed, the following applies:
+//! * Binary operators are positioned between their operands, e.g., `4 ^ 5`.
+//! * Unary operators are positioned in front of their operands, e.g., `-1` or `sin(4)`. Note that `sin4` 
+//! is parsed as variable name, but  `sin 4` is equivalent to `sin(4)`. 
+//! * Constant operators are handled as if they were numbers and are replaced by their numeric values during parsing. 
+//! They can be used as in `sin(PI)` or `4 + E`. Note that the calling notation of constant operators such as `PI()` is invalid.
+//!
+//! Binary, unary, and constant operators can be created with the functions [`make_bin`](Operator::make_bin), [`make_unary`](Operator::make_unary), 
+//! and [`make_constant`](Operator::make_constant), respectively.
+//! Operators need to be created by factories to make serialization via [`serde`](https://serde.rs/) possible as
 //! shown in the following.
 //! ```rust
 //! # use std::error::Error;
@@ -81,18 +97,25 @@
 //! ops_factory!(
 //!     IntegerOpsFactory,  // name of the factory type
 //!     i32,                // data type of the operands
-//!     Operator {
-//!         repr: "%",
-//!         bin_op: Some(BinOp{ apply: |a, b| a % b, prio: 1 }),
-//!         unary_op: None,
-//!         },
-//!     Operator {
-//!         repr: "/",
-//!         bin_op: Some(BinOp{ apply: |a, b| a / b, prio: 1 }),
-//!         unary_op: None,
-//!     }
+//!     Operator::make_bin(
+//!         "%",
+//!         BinOp{ 
+//!             apply: |a, b| a % b, 
+//!             prio: 1, 
+//!             is_commutative: false,
+//!         }
+//!     ),
+//!     Operator::make_bin(
+//!         "/",
+//!         BinOp{ 
+//!             apply: |a, b| a / b, 
+//!             prio: 1, 
+//!             is_commutative: false, 
+//!         }
+//!     ),
+//!     Operator::make_constant("TWO", 2)
 //! );
-//! let to_be_parsed = "19 % 5 / 2 / a";
+//! let to_be_parsed = "19 % 5 / TWO / a";
 //! let expr = FlatEx::<_, IntegerOpsFactory>::from_str(to_be_parsed)?;
 //! assert_eq!(expr.eval(&[1])?, 2);
 //! #
@@ -100,26 +123,22 @@
 //! # }
 //! ```
 //!
-//! To extend an existing list of operators, the macro [`ops_factory`](ops_factory) is not 
-//! sufficient. In this case one has to create a factory struct and implement the 
+//! To extend an existing list of operators, the macro [`ops_factory`](ops_factory) is not
+//! sufficient. In this case one has to create a factory struct and implement the
 //! [`MakeOperators`](MakeOperators) trait with a little boilerplate code.
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::prelude::*;
-//! use exmex::{BinOp, DefaultOpsFactory, MakeOperators, Operator};
+//! use exmex::{DefaultOpsFactory, MakeOperators, Operator};
 //! #[derive(Clone)]
 //! struct ExtendedOpsFactory;
 //! impl MakeOperators<f32> for ExtendedOpsFactory {
 //!     fn make<'a>() -> Vec<Operator<'a, f32>> {
 //!         let mut ops = DefaultOpsFactory::<f32>::make();
 //!         ops.push(
-//!             Operator {
-//!                 repr: "invert",
-//!                 bin_op: None,
-//!                 unary_op: Some(|a: f32| 1.0 / a),
-//!             },
+//!             Operator::make_unary("invert", |a| 1.0 / a)
 //!         );
 //!         ops
 //!     }
@@ -137,7 +156,7 @@
 //! [`FromStr`](std::str::FromStr). In case the representation of your data type in the
 //! string does not match the number regex `r"\.?[0-9]+(\.[0-9]+)?"`, you have to pass a
 //! suitable regex and use the function
-//! [`from_pattern`](Express::from_pattern) instead of
+//! [`from_pattern`](Express::from_pattern) instead of [`parse`](crate::parse) or
 //! [`from_str`](Express::from_str). Here is an example for `bool`.
 //! ```rust
 //! # use std::error::Error;
@@ -146,23 +165,25 @@
 //! use exmex::prelude::*;
 //! use exmex::{BinOp, MakeOperators, Operator, ops_factory};
 //! ops_factory!(
-//!     BooleanOpsFactory, 
+//!     BooleanOpsFactory,
 //!     bool,
-//!     Operator {
-//!         repr: "&&",
-//!         bin_op: Some(BinOp{ apply: |a, b| a && b, prio: 1 }),
-//!         unary_op: None,
-//!     },
-//!     Operator {
-//!         repr: "||",
-//!         bin_op: Some(BinOp{ apply: |a, b| a || b, prio: 1 }),
-//!         unary_op: None,
-//!     },
-//!     Operator {
-//!         repr: "!",
-//!         bin_op: None,
-//!         unary_op: Some(|a| !a),
-//!     }
+//!     Operator::make_bin(
+//!         "&&",
+//!         BinOp{ 
+//!             apply: |a, b| a && b, 
+//!             prio: 1, 
+//!             is_commutative: true,
+//!         }
+//!     ),
+//!     Operator::make_bin(
+//!         "||",
+//!         BinOp{ 
+//!             apply: |a, b| a || b, 
+//!             prio: 1, 
+//!             is_commutative: true, 
+//!         }
+//!     ),
+//!     Operator::make_unary("!", |a| !a)
 //! );
 //! let to_be_parsed = "!(true && false) || (!false || (true && false))";
 //! let expr = FlatEx::<_, BooleanOpsFactory>::from_pattern(to_be_parsed, "true|false")?;
@@ -252,17 +273,18 @@
 //!
 //! ## Display
 //!
-//! An instance of [`FlatEx`](FlatEx) can be displayed as string. Note that this
-//! [`unparse`](FlatEx::unparse)d string does not necessarily coincide with the original
-//! string, since, e.g., curly brackets are added and expressions are compiled.
+//! Instances of [`FlatEx`](FlatEx) and [`OwnedFlatEx`](OwnedFlatEx) can be displayed as string. Note that this
+//! [`unparse`](Express::unparse)d string does not necessarily coincide with the original
+//! string, since, e.g., curly brackets are added, expressions are compiled, and constants are
+//! replaced by their numeric values during parsing.
 //!
 //! ```rust
 //! # use std::error::Error;
 //! # fn main() -> Result<(), Box<dyn Error>> {
 //! #
 //! use exmex::prelude::*;
-//! let expr = exmex::parse::<f64>("-sin(z)/cos(mother_of_names) + 2^7")?;
-//! assert_eq!(format!("{}", expr), "-(sin({z}))/cos({mother_of_names})+128.0");
+//! let expr = exmex::parse::<f64>("-sin(z)/cos(mother_of_names) + 2^7 + E")?;
+//! assert_eq!(format!("{}", expr), "-(sin({z}))/cos({mother_of_names})+130.71828182845906");
 //! #
 //! #     Ok(())
 //! # }
@@ -308,7 +330,7 @@ pub mod prelude {
     pub use super::expression::{flat::FlatEx, Express};
 }
 
-/// Parses a string, evaluates a string, and returns the resulting number.
+/// Parses a string, evaluates the expression, and returns the resulting number.
 ///
 /// # Errrors
 ///
@@ -345,6 +367,7 @@ mod tests {
     use rand::Rng;
     use smallvec::{smallvec, SmallVec};
 
+    use crate::expression::deep::DeepEx;
     use crate::prelude::*;
     use crate::{
         eval_str,
@@ -387,19 +410,15 @@ mod tests {
             ops_factory!(
                 BitwiseOpsFactory,
                 u32,
-                Operator {
-                    repr: "|",
-                    bin_op: Some(BinOp {
+                Operator::make_bin(
+                    "|",
+                    BinOp {
                         apply: |a, b| a | b,
                         prio: 0,
-                    }),
-                    unary_op: None,
-                },
-                Operator {
-                    repr: "!",
-                    bin_op: None,
-                    unary_op: Some(|a| !a),
-                }
+                        is_commutative: true
+                    }
+                ),
+                Operator::make_unary("!", |a| !a)
             );
             let expr = FlatEx::<_, BitwiseOpsFactory>::from_str("!(a|b)")?;
             let result = expr.eval(&[0, 1])?;
@@ -546,16 +565,8 @@ mod tests {
         impl MakeOperators<f32> for SomeF32Operators {
             fn make<'a>() -> Vec<Operator<'a, f32>> {
                 vec![
-                    Operator {
-                        repr: "invert",
-                        bin_op: None,
-                        unary_op: Some(|a: f32| 1.0 / a),
-                    },
-                    Operator {
-                        repr: "sqrt",
-                        bin_op: None,
-                        unary_op: Some(|a: f32| a.sqrt()),
-                    },
+                    Operator::make_unary("invert", |a| 1.0 / a),
+                    Operator::make_unary("sqrt", |a| a.sqrt()),
                 ]
             }
         }
@@ -570,32 +581,23 @@ mod tests {
         impl MakeOperators<f32> for SomeF32Operators {
             fn make<'a>() -> Vec<Operator<'a, f32>> {
                 vec![
-                    Operator {
-                        repr: "**",
-                        bin_op: Some(BinOp {
+                    Operator::make_bin(
+                        "**",
+                        BinOp {
                             apply: |a: f32, b| a.powf(b),
                             prio: 2,
-                        }),
-                        unary_op: None,
-                    },
-                    Operator {
-                        repr: "*",
-                        bin_op: Some(BinOp {
+                            is_commutative: false
+                        },
+                    ),
+                    Operator::make_bin(
+                        "*",
+                        BinOp {
                             apply: |a, b| a * b,
                             prio: 1,
-                        }),
-                        unary_op: None,
-                    },
-                    Operator {
-                        repr: "invert",
-                        bin_op: None,
-                        unary_op: Some(|a: f32| 1.0 / a),
-                    },
-                    Operator {
-                        repr: "None",
-                        bin_op: None,
-                        unary_op: None,
-                    },
+                            is_commutative: true
+                        },
+                    ),
+                    Operator::make_unary("invert", |a: f32| 1.0 / a),
                 ]
             }
         }
@@ -603,21 +605,19 @@ mod tests {
         let val = expr.eval(&[]).unwrap();
         assert_float_eq_f32(val, 4.0 / 3.0);
 
-        let expr = FlatEx::<f32, SomeF32Operators>::from_str("None(5)");
-        assert!(expr.is_err());
-
         #[derive(Clone)]
         struct ExtendedF32Operators;
         impl MakeOperators<f32> for ExtendedF32Operators {
             fn make<'a>() -> Vec<Operator<'a, f32>> {
-                let zero_mapper = Operator {
-                    repr: "zer0",
-                    bin_op: Some(BinOp {
+                let zero_mapper = Operator::make_bin_unary(
+                    "zer0",
+                    BinOp {
                         apply: |_: f32, _| 0.0,
                         prio: 2,
-                    }),
-                    unary_op: Some(|_| 0.0),
-                };
+                        is_commutative: true
+                    },
+                    |_| 0.0,
+                );
                 DefaultOpsFactory::<f32>::make()
                     .iter()
                     .cloned()
@@ -803,35 +803,35 @@ mod tests {
         assert_float_eq_f64(eval_str("-1*(1.3+0.7*(2-1/10))").unwrap(), -2.63);
         assert_float_eq_f64(eval_str("-1*(1.3+(-0.7)*(2-1/10))").unwrap(), 0.03);
         assert_float_eq_f64(eval_str("-1*((1.3+0.7)*(2-1/10))").unwrap(), -3.8);
-        assert_float_eq_f64(eval_str("sin(3.14159265358979)").unwrap(), 0.0);
+        assert_float_eq_f64(eval_str("sin 3.14159265358979").unwrap(), 0.0);
         assert_float_eq_f64(eval_str("0-sin(3.14159265358979 / 2)").unwrap(), -1.0);
         assert_float_eq_f64(eval_str("-sin(3.14159265358979 / 2)").unwrap(), -1.0);
-        assert_float_eq_f64(eval_str("3-(-1+sin(1.5707963267948966)*2)").unwrap(), 2.0);
+        assert_float_eq_f64(eval_str("3-(-1+sin(PI/2)*2)").unwrap(), 2.0);
         assert_float_eq_f64(
             eval_str("3-(-1+sin(cos(-3.14159265358979))*2)").unwrap(),
             5.6829419696157935,
         );
         assert_float_eq_f64(
-            eval_str("-(-1+((-3.14159265358979)/5)*2)").unwrap(),
+            eval_str("-(-1+((-PI)/5)*2)").unwrap(),
             2.256637061435916,
         );
         assert_float_eq_f64(eval_str("((2-4)/5)*2").unwrap(), -0.8);
-        assert_float_eq_f64(eval_str("-(-1+(sin(-3.14159265358979)/5)*2)").unwrap(), 1.0);
+        assert_float_eq_f64(eval_str("-(-1+(sin(-PI)/5)*2)").unwrap(), 1.0);
         assert_float_eq_f64(
-            eval_str("-(-1+sin(cos(-3.14159265358979)/5)*2)").unwrap(),
+            eval_str("-(-1+sin(cos(-PI)/5)*2)").unwrap(),
             1.3973386615901224,
         );
-        assert_float_eq_f64(eval_str("-cos(3.14159265358979)").unwrap(), 1.0);
+        assert_float_eq_f64(eval_str("-cos(PI)").unwrap(), 1.0);
         assert_float_eq_f64(
-            eval_str("1+sin(-cos(-3.14159265358979))").unwrap(),
+            eval_str("1+sin(-cos(-PI))").unwrap(),
             1.8414709848078965,
         );
         assert_float_eq_f64(
-            eval_str("-1+sin(-cos(-3.14159265358979))").unwrap(),
+            eval_str("-1+sin(-cos(-PI))").unwrap(),
             -0.1585290151921035,
         );
         assert_float_eq_f64(
-            eval_str("-(-1+sin(-cos(-3.14159265358979)/5)*2)").unwrap(),
+            eval_str("-(-1+sin(-cos(-PI)/5)*2)").unwrap(),
             0.6026613384098776,
         );
         assert_float_eq_f64(eval_str("sin(-(2))*2").unwrap(), -1.8185948536513634);
@@ -880,4 +880,34 @@ mod tests {
         let deserialized = serde_json::from_str::<FlatEx<f64>>(serialized.as_str()).unwrap();
         assert_eq!(s, format!("{}", deserialized));
     }
+    #[test]
+    fn test_constants() {
+        assert_eq!(eval_str::<f64>("PI").unwrap(), std::f64::consts::PI);
+        assert_eq!(eval_str::<f64>("E").unwrap(), std::f64::consts::E);
+        let expr = parse::<f64>("x / PI * 180").unwrap();
+        assert_float_eq_f64(expr.eval(&[std::f64::consts::FRAC_PI_2]).unwrap(), 90.0);
+        
+        let expr = parse::<f32>("E ^ x").unwrap();
+        assert_float_eq_f32(expr.eval(&[5.0]).unwrap(), 1f32.exp().powf(5.0));
+
+        let expr = parse::<f32>("E ^ Erwin");
+        assert_eq!(expr.unwrap().unparse().unwrap(), "2.7182817^{Erwin}");
+    }
+
+    #[test]
+    fn test_compile() {
+        let expr = DeepEx::<f64>::from_str_float("1.0 * 3 * 2 * x / 2 / 3").unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 2.0);
+        let expr = DeepEx::<f64>::from_str_float("x*0.2*5/4+x*2*4*1*1*1*1*1*1*1+2+3+7*sin(y)-z/sin(3.0/2/(1-x*4*1*1*1*1))").unwrap();
+        assert_eq!("{x}*0.25+{x}*8.0+5.0+7.0*sin({y})-{z}/sin(1.5/(1.0-{x}*4.0))", expr.unparse_raw());
+        let expr = DeepEx::<f64>::from_str_float("x + 1 - 2").unwrap();
+        assert_float_eq_f64(expr.eval(&[0.0]).unwrap(), -1.0);
+        let expr = DeepEx::<f64>::from_str_float("x - 1 + 2").unwrap();
+        assert_float_eq_f64(expr.eval(&[0.0]).unwrap(), 1.0);
+        let expr = DeepEx::<f64>::from_str_float("x * 2 / 3").unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 4.0/3.0);
+        let expr = DeepEx::<f64>::from_str_float("x / 2 / 3").unwrap();
+        assert_float_eq_f64(expr.eval(&[2.0]).unwrap(), 1.0/3.0);
+
+    }
 }
